@@ -14,20 +14,27 @@ urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 def get_git_info(git_config, remote):
     config = configparser.ConfigParser()
     config.read(git_config)
-    if "remote \"origin\"" in config.sections():
-        url = config[f'remote "{remote}"']['url']
-        if url.startswith("git@"):
-            matches = re.match(string=url, pattern=r"git@[^:]+:([\w\-\/]+).git")
-            repository_name = matches[1]
-        else:
-            matches = re.match(string=url, pattern=r"http[s]{0,1}:\/\/[^\/]+\/([\w\-\/]+).git")
-            repository_name = matches[1]
-        
-        return repository_name
+    try:
+        # expected format of the URLs are like this
+        # git@gitlab.stonebranch.com:integration-prototypes/ue-talend-cloud.git
+        # https://gitlab.stonebranch.com/integration-prototypes/ue-talend-cloud.git
+        if "remote \"origin\"" in config.sections():
+            url = config[f'remote "{remote}"']['url']
+            if url.startswith("git@"):
+                matches = re.match(string=url, pattern=r"git@[^:]+:([\w\-\/]+).git")
+                repository_name = matches[1]
+            else:
+                matches = re.match(string=url, pattern=r"http[s]{0,1}:\/\/[^\/]+\/([\w\-\/]+).git")
+                repository_name = matches[1]
+            
+            return repository_name
+    except TypeError:
+        cprint("There is a problem with the .git/config file. Be sure that git remote origin is correct.", "red")
+        cprint("To remove the git origin use the following command", "red")
+        cprint("git remote remove origin", "yellow")
 
     return None
-    # git@gitlab.stonebranch.com:integration-prototypes/ue-talend-cloud.git
-    # https://gitlab.stonebranch.com/integration-prototypes/ue-talend-cloud.git
+    
 
 def parse_repository_full_path(repository_name, groups, group_mapping):
     result = repository = ""
@@ -173,6 +180,7 @@ class GitLab():
         self.gl = gitlab.Gitlab(url=self.base_url, private_token=token, ssl_verify=ssl_verify, per_page=100)
         self.project_id = None
         self.project = None
+        self.default_branch = None
         self.repository_name = None
         
     def get_groups(self, summary=True):
@@ -209,8 +217,23 @@ class GitLab():
         
         return projects
 
-    def create_project(self, name, group_id):
+    def create_project(self, name, group_id, config=None):
         self.project = self.gl.projects.create({"name": name, "namespace_id": group_id})
+        if config is not None:
+            if "protected_branch" in config:
+                self.log.info("Creating protected branch for the repository.")
+                protected_branch = config.get("protected_branch", None)
+                if protected_branch is not None:
+                    result = self.create_protected_branch(protected_branch)
+                    if result is False:
+                        cprint("Failed to create projected branch", "red")
+            
+            if "default_branch" in config:
+                branch_name = config.get("default_branch", None)
+                if branch_name is not None:
+                    self.default_branch = branch_name
+                    self.log.info(f"Creating default branch {branch_name}.")
+                    self.project.branches.create({ "branch": branch_name, "ref": "main" })
     
     def create_webhook(self, jenkins_url):
         if self.project is None:
@@ -229,6 +252,55 @@ class GitLab():
         
         self.hook = self.project.hooks.create({"url": jenkins_url, "push_events": True, "merge_requests_events": True, "enable_ssl_verification": True, "note_events": True, "token":"e88b637a31ca6ec4f107bb54a716d2c9"})
         return True
+    
+    def create_badge(self, jenkins_url, jenkins_base_url, jnks_group_name, jnks_repository_name):
+        if self.project is None:
+            if self.project_id is not None:
+                self.project = self.gl.projects.get(self.project_id)
+        
+        if self.project is None:
+            self.log.error("GitLab Project not found.")
+            return False
+        
+        link_url = jenkins_url + "/lastBuild/"
+        image_url = f"{jenkins_base_url}/buildStatus/icon?subject=jenkins&job={jnks_group_name}%2F{jnks_repository_name}"
+        badges = self.project.badges.list()
+        for badge in badges:
+            if badge.link_url == link_url:
+                self.log.warning("GitLab Badge for that URL is already exists.")
+                return False
+        
+        self.badge = self.project.badges.create({'link_url': link_url, 'image_url': image_url})
+        return True
+    
+    def create_protected_branch(self, protected_branch):
+        branch_name = protected_branch.get("name", None)
+        if branch_name is None:
+            cprint("Protected branch name is missing!", "red")
+            return False
+        
+        gitlab_access_level = {
+            "DEVELOPER": gitlab.const.AccessLevel.DEVELOPER,
+            "MAINTAINER": gitlab.const.AccessLevel.MAINTAINER,
+            "ADMIN": gitlab.const.AccessLevel.ADMIN,
+            "GUEST": gitlab.const.AccessLevel.GUEST,
+            "NO_ACCESS": gitlab.const.AccessLevel.NO_ACCESS,
+            "OWNER": gitlab.const.AccessLevel.OWNER,
+            "REPORTER": gitlab.const.AccessLevel.REPORTER
+        }
+        merge_config = protected_branch.get("merge", "DEVELOPER")
+        merge_level = gitlab_access_level[merge_config]
+
+        push_config = protected_branch.get("push", "MAINTAINER")
+        push_level = gitlab_access_level[push_config]
+
+        self.project.protectedbranches.create({
+            'name': branch_name,
+            'merge_access_level': merge_level,
+            'push_access_level': push_level
+        })
+
+        return True
 
 
 class Jenkins():
@@ -272,7 +344,7 @@ class Jenkins():
       <options/>
     </org.jenkinsci.plugins.pipeline.modeldefinition.actions.DeclarativeJobPropertyTrackerAction>
   </actions>
-  <description>{description}</description>
+  <description></description>
   <keepDependencies>false</keepDependencies>
   <properties>
     <org.jenkinsci.plugins.workflow.job.properties.DisableConcurrentBuildsJobProperty>
@@ -284,12 +356,34 @@ class Jenkins():
     </com.coravy.hudson.plugins.github.GithubProjectProperty>
     <com.dabsquared.gitlabjenkins.connection.GitLabConnectionProperty plugin="gitlab-plugin@1.5.22">
       <gitLabConnection>jenkins@gitlab</gitLabConnection>
-      <jobCredentialId>cs_prototype_api</jobCredentialId>
-      <useAlternativeCredential>false</useAlternativeCredential>
+      <jobCredentialId>cs_jenkins_gitlab</jobCredentialId>
+      <useAlternativeCredential>true</useAlternativeCredential>
     </com.dabsquared.gitlabjenkins.connection.GitLabConnectionProperty>
     <org.jenkinsci.plugins.gitlablogo.GitlabLogoProperty plugin="gitlab-logo@1.0.5">
       <repositoryName>{repository}</repositoryName>
     </org.jenkinsci.plugins.gitlablogo.GitlabLogoProperty>
+    <hudson.model.ParametersDefinitionProperty>
+      <parameterDefinitions>
+        <hudson.model.StringParameterDefinition>
+          <name>SourceBranch</name>
+          <description>Gitlab branch to build.</description>
+          <defaultValue>$gitlabSourceBranch</defaultValue>
+          <trim>false</trim>
+        </hudson.model.StringParameterDefinition>
+        <hudson.model.StringParameterDefinition>
+          <name>SourceRepoHomepage</name>
+          <description>{gitlab_url}/{repository}</description>
+          <defaultValue>$gitlabSourceRepoHomepage</defaultValue>
+          <trim>false</trim>
+        </hudson.model.StringParameterDefinition>
+        <hudson.model.BooleanParameterDefinition>
+          <name>DeployOnSendSafely</name>
+          <description>Flag to indicate whether the built package will be deployed on Send Safely.
+    Should only be set to true for final build on main branch.</description>
+          <defaultValue>false</defaultValue>
+        </hudson.model.BooleanParameterDefinition>
+      </parameterDefinitions>
+    </hudson.model.ParametersDefinitionProperty>
     <org.jenkinsci.plugins.workflow.job.properties.PipelineTriggersJobProperty>
       <triggers>
         <com.dabsquared.gitlabjenkins.GitLabPushTrigger plugin="gitlab-plugin@1.5.22">
@@ -299,8 +393,8 @@ class Jenkins():
           <triggerOnMergeRequest>true</triggerOnMergeRequest>
           <triggerOnlyIfNewCommitsPushed>true</triggerOnlyIfNewCommitsPushed>
           <triggerOnPipelineEvent>false</triggerOnPipelineEvent>
-          <triggerOnAcceptedMergeRequest>true</triggerOnAcceptedMergeRequest>
-          <triggerOnClosedMergeRequest>true</triggerOnClosedMergeRequest>
+          <triggerOnAcceptedMergeRequest>false</triggerOnAcceptedMergeRequest>
+          <triggerOnClosedMergeRequest>false</triggerOnClosedMergeRequest>
           <triggerOnApprovedMergeRequest>false</triggerOnApprovedMergeRequest>
           <triggerOpenMergeRequestOnPush>never</triggerOpenMergeRequestOnPush>
           <triggerOnNoteRequest>true</triggerOnNoteRequest>
@@ -332,7 +426,7 @@ class Jenkins():
       </userRemoteConfigs>
       <branches>
         <hudson.plugins.git.BranchSpec>
-          <name>origin/$gitlabSourceBranch</name>
+          <name>origin/$SourceBranch</name>
         </hudson.plugins.git.BranchSpec>
       </branches>
       <doGenerateSubmoduleConfigurations>false</doGenerateSubmoduleConfigurations>
