@@ -327,12 +327,12 @@ def dump_template_fields(_json):
     set_if_not_equal(result, "name", _json["name"], None)
     set_if_not_equal(result, "template_type", _json["templateType"], None)
     set_if_not_equal(result, "agent_type", _json["agentType"], None)
-    set_if_not_equal(result, "description", _json["description"], None)
     set_if_not_equal(result, "extension", _json["extension"], None)
     set_if_not_equal(result, "min_release", _json["minReleaseLevel"], None)
     set_if_not_equal(result, "var_prefix", _json["variablePrefix"], None)
     set_if_not_equal(result, "sys_id", _json["sysId"], None)
     set_if_not_equal(result, "script_ext_windows", _json["scriptTypeWindows"], None)
+    result["description"] = "" if _json["description"] is None else _json["description"]  # always show description
     return result
 
 def dump_events(events_json):
