@@ -27,7 +27,7 @@ import quip.field_builder as fb
 import quip.version_builder as vb
 import quip.external as external
 from argparse import RawTextHelpFormatter
-from quip import __version__, yes_or_no, cprint, choose_one
+from quip import __version__, yes_or_no, cprint, choose_one, color_text
 import keyring
 
 version = __version__
@@ -48,8 +48,6 @@ class Quip:
         logging.basicConfig(level=log_level)
         self.in_project_folder = False
         self.args = self.parse_arguments()
-        if self.args.debug:
-            logging.getLogger().setLevel(logging.DEBUG)
         self.set_global_configs(self.args.name, self.args.config)
 
     def parse_arguments(self):
@@ -105,7 +103,7 @@ class Quip:
         parser_upload.add_argument('--template', '-t', action='store_true',
                              help='create template instead of extension')
 
-        parser_download = subparsers.add_parser('download', aliases=DOWNLOAD_ACTION[1:], help='Download the template from Universal Controller. (Template Only)')
+        parser_download = subparsers.add_parser('download', aliases=DOWNLOAD_ACTION[1:], help='Download the template from Universal Controller.')
         parser_download.add_argument('name', nargs="?", help='name of the project')
         parser_download.add_argument('--template', '-t', action='store_true',
                              help='create template instead of extension')
@@ -127,13 +125,15 @@ class Quip:
         parser_setup.add_argument('name', nargs="?", help='name of the project')
 
         parser_version = subparsers.add_parser('version', help='shows the version of the template/extension')
-        parser_version.add_argument('version_method', nargs="?", choices=["minor", "major", "release"], help='update the version of the project. Options: minor,major,release.')
+        parser_version.add_argument('version_method', nargs="?", choices=["minor", "major", "release", "beta", "rc"], help='update the version of the project. Options: beta,minor,major,release,rc.')
         parser_version.add_argument('--force', dest="forced_version", help='Force to change the version in all possible files')
 
         parser_config = subparsers.add_parser('config', help='show the configuration')
 
         args = parser.parse_args()
         print(args)
+        if args.debug:
+            logging.getLogger().setLevel(logging.DEBUG)
 
         if args.action is None:
             parser.print_help()
@@ -159,13 +159,20 @@ class Quip:
                 self.in_project_folder = True
             else:
                 logging.error("You are not in a project folder. Please specify the project name.")
+                sys.exit(1)
 
         if "template" not in dir(args):
             args.template = False
+            logging.debug("No template keyword in args")
         elif args.template is False and args.name.startswith("ut-"):
             args.template = True
+            logging.debug("Project is a template because name starts with UT")
         elif args.name.startswith("ue-"):
             args.template = False
+            logging.debug("Project is an extension because name starts with UE")
+        elif args.action in DOWNLOAD_ACTION and not self.in_project_folder:
+            args.template = True
+            logging.debug("Project is a template because download action executed with a name")
         
         logging.info(f"The project is template={args.template}")
         return args
@@ -209,6 +216,7 @@ class Quip:
             self.dump_fields(write=True)
             self.create_icon_safe()
         elif action in UPLOAD_ACTION:
+            self.update_fields_if_needed()
             if not self.args.template:
                 self.run_uip("push_all")
             else:
@@ -217,13 +225,14 @@ class Quip:
             if not self.args.template:
                 self.run_uip("pull")
             else:
-                if os.path.exists(self.project_name) or self.in_project_folder:
+                if os.path.exists(self.project_folder_name) or self.in_project_folder:
                     self.download_template()
                 else:
                     self.bootstrap_template(ask_for_upload=False)
                     self.download_template()
             self.dump_fields(write=True)
         elif action == "build":
+            self.update_fields_if_needed()
             if self.args.template:
                 self.build_zip(self.project_name)
             else:
@@ -277,31 +286,33 @@ class Quip:
         self._global_conf_external = self.uip_global_config.get("external", {})
         self._version_files = self.uip_global_config.get("version_files", None)
         self.default_template = self._global_conf_defaults.get("template", "ue-task")
+        self.project_prefix = self._global_conf_defaults.get("project_prefix", None)
         self.project_name = self.format_ext_name(project_name)
-        self.extension_name = self.format_ext_name(project_name.lower())
+        self.extension_name = self.project_folder_name = self.format_project_folder_name(project_name.lower(), self.args.template, self.project_prefix)
         self.template_name = self.titleize(project_name)
         self.use_keyring = self._global_conf_defaults.get("use_keyring", True)
         logging.debug(f"Project Name: {self.project_name}")
         logging.debug(f"Template Name: {self.template_name}")
+        logging.debug(f"Folder Name: {self.project_folder_name}")
 
     def new_template(self):
         logging.info(f"creating new template {self.template_name}")
-        if os.path.exists(self.project_name):
+        if os.path.exists(self.project_folder_name):
             logging.error("Folder already exists")
             sys.exit(1)
 
-        os.makedirs(self.project_name)
+        os.makedirs(self.project_folder_name)
         os.makedirs(self.join_path("src"))
         os.makedirs(self.join_path("src", "templates"))
 
     def new_project(self):
         logging.info(f"creating new extension {self.template_name}")
-        if os.path.exists(self.project_name):
+        if os.path.exists(self.project_folder_name):
             print("ERROR: Folder already exists")
             sys.exit(1)
         
-        os.makedirs(self.project_name)
-        self.uip_init(self.project_name, self.default_template)
+        os.makedirs(self.project_folder_name)
+        self.uip_init(self.project_folder_name, self.default_template)
         self.update_extension_yaml(self.extension_name)
         self.update_uip_config(self.project_name, new_project=True)
         self.update_template_json(self.project_name)
@@ -312,7 +323,7 @@ class Quip:
             self.update_extension_yaml(self.extension_name, new_project=new_project)
             self.update_uip_config(self.project_name, new_project=new_project)
         else:
-            self.update_script_config(self.project_name)
+            self.update_script_config(self.project_folder_name)
         self.update_template_json(self.project_name, update_uuid, update_new_uuid)
 
     def update_rename_scripts(self):
@@ -408,9 +419,16 @@ class Quip:
     
     def get_icon_message(self, message):
         result = []
+        message = message.upper()
         message = message.replace("-", " ")
-        message = message.replace("  ", " ").upper()
+        message = message.replace("  ", " ")
         words = message.split(" ")
+        if words[0] in ["UT", "UE"]:
+            words = words[1:]
+
+        if self.project_prefix is not None and words[0] == self.project_prefix.upper():
+            words = words[1:]
+
         if len(words[-1]) < 4:
             return words[-1]
         else:
@@ -426,11 +444,11 @@ class Quip:
 
     def delete_project(self):
         logging.info(f"Deleting extension {self.template_name}")
-        if not os.path.exists(self.project_name):
+        if not os.path.exists(self.project_folder_name):
             logging.error("Folder doesn't exist")
             sys.exit(1)
 
-        shutil.rmtree(self.project_name)
+        shutil.rmtree(self.project_folder_name)
     
     def clean_project(self):
         for folder in ["build", "dist", "temp", "downloads"]:
@@ -464,15 +482,16 @@ class Quip:
             logging.error("From path is not a extension path")
             sys.exit(1)
 
-        if os.path.exists(self.project_name):
+        if os.path.exists(self.project_folder_name):
             logging.error("Folder already exists")
             sys.exit(1)
         
-        os.makedirs(self.project_name)
+        os.makedirs(self.project_folder_name)
         if not self.args.template:
-            self.uip_init(self.project_name, self.default_template)
+            self.uip_init(self.project_folder_name, self.default_template)
 
         if not all_files:
+            logging.debug("Copying all files to new folder.")
             files = copy_tree(from_project_src_path, self.join_path("src"))
         else:
             source_files = os.listdir(from_project_path)
@@ -642,9 +661,11 @@ class Quip:
                 with open(template, "w") as f:
                     f.write(self.format_json(_json))
                     logging.debug("template.json file is updated")
-                
                 self.dump_fields(write=True)
                 logging.debug("fields.yml updated")
+                
+                # have modified time of template be later than fields.yml
+                os.utime(template)
             else:
                 logging.error(f"ERROR: template.json file is missing! Path= {template}")
                 sys.exit(1)
@@ -652,6 +673,14 @@ class Quip:
             logging.error(f"fields.yml file is missing")
             sys.exit(4)
 
+    def update_fields_if_needed(self):
+        template_json_path = self.join_path("src", "templates", "template.json")
+        fields_path = self.join_path("fields.yml")
+        if os.path.getmtime(fields_path) > os.path.getmtime(template_json_path):
+            cprint("It looks like fields.yml file changed.", "cyan")
+            if yes_or_no("Do you want to update template.json?", default=True, color="cyan"):
+                self.update_fields()
+
     def dump_fields(self, write=False):
         logging.info("Writing fields to fields.yml file")
         template = self.join_path("src", "templates", "template.json")
@@ -673,6 +702,9 @@ class Quip:
                     fields_path = self.join_path("fields.yml")
                     with open(fields_path, "w") as f:
                         f.write(yaml_dump)
+            
+            # Update template.json time
+            os.utime(template)
         else:
             logging.error(f"ERROR: template.json file is missing! Path= {template}")
             sys.exit(1)
@@ -713,7 +745,10 @@ class Quip:
         # create webhook for Jenkins
         if gl_config is not None and jnks_config is not None:
             jenkins_url = jnks.get_url(jnks_group_name, jnks_repository_name)
-            gl.create_webhook(jenkins_url)
+            if gl.create_webhook(jenkins_url):
+                cprint("Webhook created", "green")
+            if gl.create_badge(jenkins_url, jnks_config.get("url"), jnks_group_name, jnks_repository_name):
+                cprint("Badge created", "green")
 
         if sq_config is not None:
             cprint("\n==== SONARQUBE SETUP ====", "blue")
@@ -741,10 +776,10 @@ class Quip:
             else:
                 cprint(f"Repository doesn't exist in GitLab: {repository_name}", "red")
                 if yes_or_no("Do you want to create GitLab repository?", default=True):
-                    repository_name = self.create_gitlab(gl, repository_name)
+                    repository_name = self.create_gitlab(gl, repository_name, config=gl_config)
         else:
             if yes_or_no("Do you want to create Gitlab repository?", default=True):
-                repository_name = self.create_gitlab(gl)
+                repository_name = self.create_gitlab(gl, config=gl_config)
         
         gl.repository_name = repository_name
         return gl
@@ -778,10 +813,10 @@ class Quip:
         ssl_verify = sq_config.get("ssl_verify", True)
         sq = external.SonarQube(sq_url, (username, sq_pass), ssl_verify)
         prefix = group_name
-        sq_project_name = prefix + "-" + repository_name
+        sq_project_name = prefix + "_" + repository_name
         sq.create_project(sq_project_name)
 
-    def create_gitlab(self, gl, repository_name=None):
+    def create_gitlab(self, gl, repository_name=None, config=None):
         logging.info("Creating GitLab Repository.")
         
         if repository_name is None:
@@ -796,11 +831,31 @@ class Quip:
             cprint(f"There is a repository already exists in GitLab. ({repository_name})", "yellow")
             cprint(f"Run `git remote add origin {gl.base_url}/{repository_name}.git", "yellow")
         else:
-            gl.create_project(self.extension_name, group_id)
+            gl.create_project(self.extension_name, group_id, config=config)
             cprint(f"Repository created {repository_name}", "green")
-            cprint(f"Run `git init`", "green")
-            cprint(f"Run `git remote add origin {gl.base_url}/{repository_name}.git`", "green")
         
+            if config.get("git-init", False):
+                self.run_git("init")
+                self.run_git(f"remote add origin {gl.base_url}/{repository_name}.git")
+                if gl.default_branch is not None:
+                    if os.path.exists(self.join_path("README.md")):
+                        os.rename(self.join_path("README.md"), self.join_path("README.md.temp"))
+                    self.run_git(f"checkout -b {gl.default_branch}")
+                    self.run_git(f"pull origin {gl.default_branch}")
+                    if os.path.exists(self.join_path("README.md.temp")):
+                        os.remove(self.join_path("README.md"))
+                        os.rename(self.join_path("README.md.temp"), self.join_path("README.md"))
+
+            else:
+                cprint(f"Run `git init`", "green")
+                cprint(f"Run `git remote add origin {gl.base_url}/{repository_name}.git`", "green")
+
+                if gl.default_branch is not None:
+                    cprint(f"Default branch created. There will be an initial commit for that branch for README.md file", "yellow")
+                    cprint(f"Be sure you rename README.md file and run ", "yellow")
+                    cprint(f"     git checkout -b {gl.default_branch}", "yellow")
+                    cprint(f"     git pull origin {gl.default_branch}", "yellow")
+            
         return repository_name
     
     def check_gitlab_repository_exists(self, gl, repository_name):
@@ -841,7 +896,6 @@ class Quip:
                 logging.warn("Jenkins configuration is missing in the uip_config file.")
                 return None
 
-            
             username = jnks_config.get("username", True)
             jnks_token = self.ask_password(jnks_url, username, prompt=f"Please enter Jenkins Password for ({username}): ")
             ssl_verify = jnks_config.get("ssl_verify", True)
@@ -887,6 +941,8 @@ class Quip:
             print(f"   RELEASE: ", vb.get_new_version("release", current_versions[0]))
             print(f"   MAJOR: ", vb.get_new_version("major", current_versions[0]))
             print(f"   MINOR: ", vb.get_new_version("minor", current_versions[0]))
+            print(f"   BETA: ", vb.get_new_version("beta", current_versions[0]))
+            print(f"   RC: ", vb.get_new_version("rc", current_versions[0]))
 
     def update_version(self, method, current_version, forced_version=None):
         if forced_version is None:
@@ -915,13 +971,35 @@ class Quip:
         name = name.replace("_", "-")
         name = name.replace(" ", "-")
         name = name.replace("/", "")
+        name = name.replace("---", "-")
+        name = name.replace("--", "-")
         return name.lower()
+    
+    def format_project_folder_name(self, project_name, template, prefix=None):
+        if len(project_name) == 0:
+            return project_name
+        
+        ext_name = self.format_ext_name(project_name.lower())
+
+        if ext_name[:3] in ["ue-", "ut-"]:
+            ext_name = ext_name[3:]
+        
+        if prefix is not None:
+            if not ext_name.startswith(prefix):
+                ext_name = prefix + "-" + ext_name
+
+        if template:
+            ext_name = "ut-" + ext_name
+        else:
+            ext_name = "ue-" + ext_name
+
+        return ext_name
 
     def join_path(self, *paths):
         if self.in_project_folder:
             return os.path.join(os.getcwd(), *paths)
         else:
-            return os.path.join(self.project_name, *paths)
+            return os.path.join(self.project_folder_name, *paths)
 
     def uip_init(self, project_name, default_template):
         # run uip init command
@@ -974,6 +1052,27 @@ class Quip:
                 else:
                     print(line)
             cprint(f"==============", color="yellow")
+    
+    def run_git(self, command):
+        command = "git " + command
+        cprint(command, color="yellow")
+        result = subprocess.run(command, shell=True, stderr=subprocess.PIPE, stdout=subprocess.PIPE, universal_newlines=True)
+        if result.returncode != 0:
+            logging.error("GIT command failed.")
+            logging.error(f"ERROR: Command is {command}")
+            logging.error(f"ERROR: Return code is {command.returncode}")
+            sys.exit(command.returncode)
+        else:
+            if len(result.stdout) > 0:
+                cprint(f"======= GIT Output =======", color="yellow")
+                for line in result.stdout.splitlines():
+                    if line.lower().startswith("success"):
+                        cprint(line, color="green")
+                    elif line.lower().find("error") > 0:
+                        cprint(line, color="red")
+                    else:
+                        print(line)
+                cprint(f"==============", color="yellow")
             
     def rename_build_package(self, version):
         package_folder = self.join_path("dist", "package_build")
@@ -1075,6 +1174,11 @@ class Quip:
                 if "extension" in _json and _json["extension"] is not None:
                     _json["extension"] = self.extension_name
                 _json["name"] = self.template_name
+                if "variablePrefix" in _json:
+                    if self.args.template:
+                        _json["variablePrefix"] = "var"    
+                    else:
+                        _json["variablePrefix"] = self.extension_name.replace("-", "_")
                 f.write(self.format_json(_json))
                 logging.debug("template.json file is updated")
         else:
@@ -1232,7 +1336,8 @@ class Quip:
     def get_new_uuid(self):
         return str(uuid.uuid4()).replace("-","")
     
-    def ask_password(self, server, user_name, prompt=None):
+    def ask_password(self, server, user_name, prompt=None, color='cyan', style='normal'):
+        cprint(f"Server = `{server}`, User = `{user_name}`", color, style=style)
         if prompt is None:
             prompt = f'''Enter password for {user_name}: '''
             if self.use_keyring:
@@ -1242,6 +1347,8 @@ class Quip:
             if self.use_keyring:
                 if (keyring.get_password(server, user_name) is not None):
                     print("There is a saved password. To used the saved password just press [Enter]")
+        if color is not None:
+            prompt = color_text(prompt, color, style=style)
         password = getpass(prompt=prompt)
         if len(password) == 0:
             if self.use_keyring:
